export interface Cache {
    /**
     * Timeouts in 2000 ms and returns null on timeout,
     * you can override this behaviour by passing the options parameter
     */
    get<T>(key: string, options?: {
        responseTimeoutInMillis?: number;
        throwOnTimeout?: boolean;
    }): Promise<T | null>;
    /**
     * Timeouts in 2000 ms secs and do not throws,
     * you can override this behaviour by passing the options parameter
     */
    set(key: string, data: any, tags?: string[], options?: {
        ttlInSecs?: number;
        responseTimeoutInMillis?: number;
        throwOnTimeout?: boolean;
    }): Promise<void>;
    /**
     * Timeouts in 2000 ms secs and do not throws,
     * you can override this behaviour by passing the options parameter
     */
    set(key: string, data: any, options?: {
        ttlInSecs?: number;
        responseTimeoutInMillis?: number;
        throwOnTimeout?: boolean;
    }): Promise<void>;
    /**
     * Timeouts in 2000 ms secs and do not throws,
     * you can override this behaviour by passing the options parameter
     */
    delete(key: string, options?: {
        responseTimeoutInMillis?: number;
        throwOnTimeout?: boolean;
    }): Promise<void>;
    /**
     * Timeouts in 2000 ms secs and do not throws,
     * you can override this behaviour by passing the options parameter
     */
    invalidateTags: (tags: string[], options?: {
        responseTimeoutInMillis?: number;
        throwOnTimeout?: boolean;
    }) => Promise<void>;
}
